#ifndef GLOBAL_H
#define GLOBAL_H

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <dirent.h>


#ifdef HAS_A429_LIBRARY
#include "a429_library.h"
#endif

#ifdef HAS_EXTA429_LIBRARY
#include "a429ext_library.h"
#endif

#define VERSION_APP                     "15.02.2023"

/// \brief паттерн имени девайса
#define STR_CH0_PATTERN                 "-ch-0"

/// \brief паттерн имени канала - PCI подключение
#define STR_DEV_PCI_PATTERN             "a429dev"

/// \brief паттерн имени канала - USB подключение
#define STR_DEV_USB_PATTERN             "a429usb"

/// \brief каталог с каналами
#define STR_DEV                         "/dev"
/// \brief каталог с конфигами тестов
#define STR_CONFIG                      "./config/"
/// \brief каталог с логами
#define STR_LOGS                        "./logs/"
/// \brief каталог с mem файлами
#define STR_MEM                         "./mem/"
/// \brief длина имени каталога config
#define LEN_STR_CONFIG                  9
/// \brief длина имени config файла
#define MAX_LEN_CONFIG_FILE             90
/// \brief длина полного имени config файла
#define MAX_LEN_FULL_CONFIG_FILE        MAX_LEN_CONFIG_FILE + LEN_STR_CONFIG
/// \brief длина имени канала
#define CHANNEL_NAME_LENGTH             30
/// \brief длина имени log файла
#define LOGGER_FILENAME_LENGTH          160
/// \brief длина имени mem файла
#define MEM_FILENAME_LENGTH             160
/// \brief код значения - хорошо
#define GOOD_CODE                       0
/// \brief код значения - плохо
#define BAD_CODE                        -1

///
/// \brief The TChannel struct информация о канале
///
struct TChannel {
    char mNameChannel[CHANNEL_NAME_LENGTH];             ///< полное имя канала
    T_INFO_CHANNEL mInfo;                               ///< инфо о канале
};

///
/// \brief The TDevice struct информация о девайсе
///
struct TDevice {
    char mNameChannel[CHANNEL_NAME_LENGTH];             ///< полное имя канала
    unsigned int mDevId;                                ///< device id
};

#ifdef HAS_EXTA429_LIBRARY

/// Задать сетевое устройство
bool setNetworkDevice(const char *device);

#endif // HAS_EXTA429_LIBRARY

///
/// \brief getCountDevices получить количество девайсов
/// \return кол-во девайсов
///
int getCountDevices(void);
///
/// \brief getCountChannels получить кол-во каналов ДПК
/// \return кол-во каналов
///
int getCountChannels(void);
///
/// \brief getArrayDevices получить массив описателей девайсов
/// \param devices массив описателей девайсов
/// \param length длина массива
///
void getArrayDevices(struct TDevice** devices, int* length);
///
/// \brief getArrayChannels получить массив описателей каналов ДПК
/// \param channels массив описателей каналов
/// \param length длина массива
///
void getArrayChannels(struct TChannel** channels, int* length);
///
/// \brief readline чтение строки
/// \param stream указатель текущего положения в файле
/// \param line указатель на строку
///
void readline(FILE* stream, char** line);
///
/// \brief constructLogFilename сгенерить имя лог файла
/// \param filename полное имя лог файла
/// \param length длина
/// \param testname имя теста
///
void constructLogFilename(char* filename, int length, char* testname);
///
/// \brief hasDmaBlockA429RcvErr проверка наличия ошибок в блоке дма примника а429
/// \param block блок дма
/// \return 1 - есть, 0 - не
///
int hasDmaBlockA429RcvErr(T_BLOCK_DMA* block);
///
/// \brief printErrA429Rcv печать ошибок из блоков дма приёмника а429
/// \param ch канал
/// \param dma контейнер дма
///
void printErrA429Rcv(struct TChannel* ch, T_CONTAINER_DMA* dma);


/// \remark LOGGER

///
/// \brief LOGGER_FILE дескриптор файла логов
///
extern FILE* LOGGER_FILE;
/// \brief открыть лог файл
#define LOGGER_OPEN(filename) \
    LOGGER_FILE = fopen(filename, "w");
/// \brief закрыть лог файл
#define LOGGER_CLOSE() \
    fclose(LOGGER_FILE)
/// \brief сбросить лог файл буфер на диск
#define LOGGER_FLUSH() \
    fflush(LOGGER_FILE)
/// \brief принт в лог и в терминал
#define LOGGER_PRINT(fmt, args...) \
    printf(fmt, args); \
    fprintf(LOGGER_FILE, fmt, args)

#ifndef max
#define max(x, y) (((x) > (y)) ? (x) : (y))
#endif

#ifndef min
#define min(x, y) (((x) < (y)) ? (x) : (y))
#endif


typedef struct
{
    char name[ CHANNEL_NAME_LENGTH ];
    bool isExt; // режим взаимодействия (true для внешнего устройства, с библиотекой сокетов)

#ifdef HAS_A429_LIBRARY
    int fd; // дескриптор канала (при взаимодействии с драйвером)
#endif

#ifdef HAS_EXTA429_LIBRARY
    A429ChannelFD extFd; // дескриптор канала (при взаимодействии с сокетом)
#endif

} Abonent;


#if defined HAS_A429_LIBRARY && defined HAS_EXTA429_LIBRARY

/**
 \brief обёртка для точно такого же вызова, но с приставкой ext, для внешних устройств
 \param abonent структура "абонент"
 \param FUNC название команды для выполнения, после которого следуют аргументы, передаваемые команде
*/
#define IS_EXT_WRAPPER(abonent, FUNC, ...) \
    ((abonent.isExt) ? ext##FUNC( abonent.extFd, ##__VA_ARGS__ ) : FUNC( abonent.fd, ##__VA_ARGS__ ))

#else

#ifdef HAS_A429_LIBRARY

#define IS_EXT_WRAPPER(abonent, FUNC, ...) \
    FUNC( abonent.fd, ##__VA_ARGS__ )

#else
#ifdef HAS_EXTA429_LIBRARY
#define IS_EXT_WRAPPER(abonent, FUNC, ...) \
    ext##FUNC( abonent.extFd, ##__VA_ARGS__ )

#endif
#endif
#endif

/// Элемент кэша абонентов
struct AbonentItem
{
    Abonent abonent; ///< абонент
    struct AbonentItem *next; ///< следующий элемент
};

/// Получить открытый дескриптор по имени устройства (результат кэшируется)
Abonent open_abonent_from_string(char *path);

/// Закрыть все дескрипторы из кэша
void close_all_abonents();

#endif // GLOBAL_H
